@@ -52,13 +52,13 @@ else:
             self.size_limit = size_limit
             super().__init__()
 
-        def __setitem__(self, __key: Any, __value: Any) -> None:
-            super().__setitem__(__key, __value)
+        def __setitem__(self, key: Any, value: Any, /) -> None:
+            super().__setitem__(key, value)
             if len(self) > self.size_limit:
                 excess = len(self) - self.size_limit + self.size_limit // 10
                 to_remove = list(self.keys())[:excess]
-                for key in to_remove:
-                    del self[key]
+                for k in to_remove:
+                    del self[k]
 
 
 # weak dictionaries allow the dynamically created parametrized versions of generic models to get collected
